# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::MobileEngagement::Mgmt::V2014_12_01
  #
  # Microsoft Azure Mobile Engagement REST APIs.
  #
  class ExportTasks
    include MsRestAzure

    #
    # Creates and initializes a new instance of the ExportTasks class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [EngagementManagementClient] reference to the EngagementManagementClient
    attr_reader :client

    #
    # Get the list of export tasks.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param skip [Integer] Control paging of export tasks, start results at the
    # given offset, defaults to 0 (1st page of data).
    # @param top [Integer] Control paging of export tasks, number of export tasks
    # to return with each call. By default, it returns all export tasks with a
    # default paging of 20.
    # The response contains a `nextLink` property describing the path to get the
    # next page if there are more results.
    # The maximum paging limit for $top is 40.
    # @param orderby [String] Sort results by an expression which looks like
    # `$orderby=taskId asc` (default when not specified).
    # The syntax is orderby={property} {direction} or just orderby={property}.
    # Properties that can be specified for sorting: taskId, errorDetails,
    # dateCreated, taskStatus, and dateCreated.
    # The available directions are asc (for ascending order) and desc (for
    # descending order).
    # When not specified the asc direction is used.
    # Only one orderby property can be specified.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<ExportTaskResult>] operation results.
    #
    def list(resource_group_name, app_collection, app_name, skip:0, top:20, orderby:nil, custom_headers:nil)
      first_page = list_as_lazy(resource_group_name, app_collection, app_name, skip:skip, top:top, orderby:orderby, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Get the list of export tasks.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param skip [Integer] Control paging of export tasks, start results at the
    # given offset, defaults to 0 (1st page of data).
    # @param top [Integer] Control paging of export tasks, number of export tasks
    # to return with each call. By default, it returns all export tasks with a
    # default paging of 20.
    # The response contains a `nextLink` property describing the path to get the
    # next page if there are more results.
    # The maximum paging limit for $top is 40.
    # @param orderby [String] Sort results by an expression which looks like
    # `$orderby=taskId asc` (default when not specified).
    # The syntax is orderby={property} {direction} or just orderby={property}.
    # Properties that can be specified for sorting: taskId, errorDetails,
    # dateCreated, taskStatus, and dateCreated.
    # The available directions are asc (for ascending order) and desc (for
    # descending order).
    # When not specified the asc direction is used.
    # Only one orderby property can be specified.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_with_http_info(resource_group_name, app_collection, app_name, skip:0, top:20, orderby:nil, custom_headers:nil)
      list_async(resource_group_name, app_collection, app_name, skip:skip, top:top, orderby:orderby, custom_headers:custom_headers).value!
    end

    #
    # Get the list of export tasks.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param skip [Integer] Control paging of export tasks, start results at the
    # given offset, defaults to 0 (1st page of data).
    # @param top [Integer] Control paging of export tasks, number of export tasks
    # to return with each call. By default, it returns all export tasks with a
    # default paging of 20.
    # The response contains a `nextLink` property describing the path to get the
    # next page if there are more results.
    # The maximum paging limit for $top is 40.
    # @param orderby [String] Sort results by an expression which looks like
    # `$orderby=taskId asc` (default when not specified).
    # The syntax is orderby={property} {direction} or just orderby={property}.
    # Properties that can be specified for sorting: taskId, errorDetails,
    # dateCreated, taskStatus, and dateCreated.
    # The available directions are asc (for ascending order) and desc (for
    # descending order).
    # When not specified the asc direction is used.
    # Only one orderby property can be specified.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_async(resource_group_name, app_collection, app_name, skip:0, top:20, orderby:nil, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'app_collection is nil' if app_collection.nil?
      fail ArgumentError, 'app_name is nil' if app_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, "'skip' should satisfy the constraint - 'InclusiveMinimum': '0'" if !skip.nil? && skip < 0
      fail ArgumentError, "'top' should satisfy the constraint - 'InclusiveMaximum': '40'" if !top.nil? && top > 40
      fail ArgumentError, "'top' should satisfy the constraint - 'InclusiveMinimum': '1'" if !top.nil? && top < 1


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MobileEngagement/appcollections/{appCollection}/apps/{appName}/devices/exportTasks'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'appCollection' => app_collection,'appName' => app_name},
          query_params: {'api-version' => @client.api_version,'$skip' => skip,'$top' => top,'$orderby' => orderby},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::ExportTaskListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Retrieves information about a previously created export task.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param id [String] Export task identifier.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ExportTaskResult] operation results.
    #
    def get(resource_group_name, app_collection, app_name, id, custom_headers:nil)
      response = get_async(resource_group_name, app_collection, app_name, id, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Retrieves information about a previously created export task.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param id [String] Export task identifier.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(resource_group_name, app_collection, app_name, id, custom_headers:nil)
      get_async(resource_group_name, app_collection, app_name, id, custom_headers:custom_headers).value!
    end

    #
    # Retrieves information about a previously created export task.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param id [String] Export task identifier.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(resource_group_name, app_collection, app_name, id, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'app_collection is nil' if app_collection.nil?
      fail ArgumentError, 'app_name is nil' if app_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'id is nil' if id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MobileEngagement/appcollections/{appCollection}/apps/{appName}/devices/exportTasks/{id}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'appCollection' => app_collection,'appName' => app_name,'id' => id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::ExportTaskResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates a task to export activities.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ExportTaskResult] operation results.
    #
    def create_activities_task(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      response = create_activities_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates a task to export activities.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_activities_task_with_http_info(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      create_activities_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
    end

    #
    # Creates a task to export activities.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_activities_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'app_collection is nil' if app_collection.nil?
      fail ArgumentError, 'app_name is nil' if app_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::DateRangeExportTaskParameter.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MobileEngagement/appcollections/{appCollection}/apps/{appName}/devices/exportTasks/activities'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'appCollection' => app_collection,'appName' => app_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 202
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 202
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::ExportTaskResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates a task to export crashes.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ExportTaskResult] operation results.
    #
    def create_crashes_task(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      response = create_crashes_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates a task to export crashes.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_crashes_task_with_http_info(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      create_crashes_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
    end

    #
    # Creates a task to export crashes.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_crashes_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'app_collection is nil' if app_collection.nil?
      fail ArgumentError, 'app_name is nil' if app_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::DateRangeExportTaskParameter.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MobileEngagement/appcollections/{appCollection}/apps/{appName}/devices/exportTasks/crashes'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'appCollection' => app_collection,'appName' => app_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 202
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 202
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::ExportTaskResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates a task to export errors.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ExportTaskResult] operation results.
    #
    def create_errors_task(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      response = create_errors_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates a task to export errors.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_errors_task_with_http_info(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      create_errors_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
    end

    #
    # Creates a task to export errors.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_errors_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'app_collection is nil' if app_collection.nil?
      fail ArgumentError, 'app_name is nil' if app_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::DateRangeExportTaskParameter.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MobileEngagement/appcollections/{appCollection}/apps/{appName}/devices/exportTasks/errors'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'appCollection' => app_collection,'appName' => app_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 202
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 202
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::ExportTaskResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates a task to export events.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ExportTaskResult] operation results.
    #
    def create_events_task(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      response = create_events_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates a task to export events.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_events_task_with_http_info(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      create_events_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
    end

    #
    # Creates a task to export events.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_events_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'app_collection is nil' if app_collection.nil?
      fail ArgumentError, 'app_name is nil' if app_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::DateRangeExportTaskParameter.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MobileEngagement/appcollections/{appCollection}/apps/{appName}/devices/exportTasks/events'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'appCollection' => app_collection,'appName' => app_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 202
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 202
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::ExportTaskResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates a task to export jobs.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ExportTaskResult] operation results.
    #
    def create_jobs_task(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      response = create_jobs_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates a task to export jobs.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_jobs_task_with_http_info(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      create_jobs_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
    end

    #
    # Creates a task to export jobs.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_jobs_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'app_collection is nil' if app_collection.nil?
      fail ArgumentError, 'app_name is nil' if app_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::DateRangeExportTaskParameter.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MobileEngagement/appcollections/{appCollection}/apps/{appName}/devices/exportTasks/jobs'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'appCollection' => app_collection,'appName' => app_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 202
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 202
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::ExportTaskResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates a task to export sessions.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ExportTaskResult] operation results.
    #
    def create_sessions_task(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      response = create_sessions_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates a task to export sessions.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_sessions_task_with_http_info(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      create_sessions_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
    end

    #
    # Creates a task to export sessions.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [DateRangeExportTaskParameter]
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_sessions_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'app_collection is nil' if app_collection.nil?
      fail ArgumentError, 'app_name is nil' if app_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::DateRangeExportTaskParameter.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MobileEngagement/appcollections/{appCollection}/apps/{appName}/devices/exportTasks/sessions'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'appCollection' => app_collection,'appName' => app_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 202
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 202
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::ExportTaskResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates a task to export tags.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [ExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ExportTaskResult] operation results.
    #
    def create_tags_task(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      response = create_tags_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates a task to export tags.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [ExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_tags_task_with_http_info(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      create_tags_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
    end

    #
    # Creates a task to export tags.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [ExportTaskParameter]
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_tags_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'app_collection is nil' if app_collection.nil?
      fail ArgumentError, 'app_name is nil' if app_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::ExportTaskParameter.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MobileEngagement/appcollections/{appCollection}/apps/{appName}/devices/exportTasks/tags'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'appCollection' => app_collection,'appName' => app_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 202
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 202
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::ExportTaskResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates a task to export tags.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [ExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ExportTaskResult] operation results.
    #
    def create_tokens_task(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      response = create_tokens_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates a task to export tags.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [ExportTaskParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_tokens_task_with_http_info(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      create_tokens_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
    end

    #
    # Creates a task to export tags.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [ExportTaskParameter]
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_tokens_task_async(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'app_collection is nil' if app_collection.nil?
      fail ArgumentError, 'app_name is nil' if app_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::ExportTaskParameter.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MobileEngagement/appcollections/{appCollection}/apps/{appName}/devices/exportTasks/tokens'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'appCollection' => app_collection,'appName' => app_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 202
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 202
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::ExportTaskResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates a task to export push campaign data for a date range.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [FeedbackByDateRangeParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ExportTaskResult] operation results.
    #
    def create_feedback_task_by_date_range(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      response = create_feedback_task_by_date_range_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates a task to export push campaign data for a date range.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [FeedbackByDateRangeParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_feedback_task_by_date_range_with_http_info(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      create_feedback_task_by_date_range_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
    end

    #
    # Creates a task to export push campaign data for a date range.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [FeedbackByDateRangeParameter]
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_feedback_task_by_date_range_async(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'app_collection is nil' if app_collection.nil?
      fail ArgumentError, 'app_name is nil' if app_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::FeedbackByDateRangeParameter.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MobileEngagement/appcollections/{appCollection}/apps/{appName}/devices/exportTasks/feedbackByDate'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'appCollection' => app_collection,'appName' => app_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 202
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 202
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::ExportTaskResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates a task to export push campaign data for a set of campaigns.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [FeedbackByCampaignParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ExportTaskResult] operation results.
    #
    def create_feedback_task_by_campaign(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      response = create_feedback_task_by_campaign_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates a task to export push campaign data for a set of campaigns.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [FeedbackByCampaignParameter]
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_feedback_task_by_campaign_with_http_info(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      create_feedback_task_by_campaign_async(resource_group_name, app_collection, app_name, parameters, custom_headers:custom_headers).value!
    end

    #
    # Creates a task to export push campaign data for a set of campaigns.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param parameters [FeedbackByCampaignParameter]
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_feedback_task_by_campaign_async(resource_group_name, app_collection, app_name, parameters, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'app_collection is nil' if app_collection.nil?
      fail ArgumentError, 'app_name is nil' if app_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::FeedbackByCampaignParameter.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MobileEngagement/appcollections/{appCollection}/apps/{appName}/devices/exportTasks/feedbackByCampaign'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'appCollection' => app_collection,'appName' => app_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 202
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 202
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::ExportTaskResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Get the list of export tasks.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ExportTaskListResult] operation results.
    #
    def list_next(next_page_link, custom_headers:nil)
      response = list_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Get the list of export tasks.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_next_with_http_info(next_page_link, custom_headers:nil)
      list_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Get the list of export tasks.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MobileEngagement::Mgmt::V2014_12_01::Models::ExportTaskListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Get the list of export tasks.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param app_collection [String] Application collection.
    # @param app_name [String] Application resource name.
    # @param skip [Integer] Control paging of export tasks, start results at the
    # given offset, defaults to 0 (1st page of data).
    # @param top [Integer] Control paging of export tasks, number of export tasks
    # to return with each call. By default, it returns all export tasks with a
    # default paging of 20.
    # The response contains a `nextLink` property describing the path to get the
    # next page if there are more results.
    # The maximum paging limit for $top is 40.
    # @param orderby [String] Sort results by an expression which looks like
    # `$orderby=taskId asc` (default when not specified).
    # The syntax is orderby={property} {direction} or just orderby={property}.
    # Properties that can be specified for sorting: taskId, errorDetails,
    # dateCreated, taskStatus, and dateCreated.
    # The available directions are asc (for ascending order) and desc (for
    # descending order).
    # When not specified the asc direction is used.
    # Only one orderby property can be specified.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ExportTaskListResult] which provide lazy access to pages of the
    # response.
    #
    def list_as_lazy(resource_group_name, app_collection, app_name, skip:0, top:20, orderby:nil, custom_headers:nil)
      response = list_async(resource_group_name, app_collection, app_name, skip:skip, top:top, orderby:orderby, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

  end
end
